@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,12 +20,16 @@ sabnzbd.config - Configuration Support
 """
 
 import os
+import re
 import logging
 import threading
 import shutil
+import time
+import random
+from hashlib import md5
+from urlparse import urlparse
 import sabnzbd.misc
-from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY
-from sabnzbd.utils import listquote
+from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY, MAX_WIN_DFOLDER
 from sabnzbd.utils import configobj
 from sabnzbd.decorators import synchronized
 
@@ -34,30 +38,35 @@ SAVE_CONFIG_LOCK = threading.Lock()
 
 
 CFG = {}                    # Holds INI structure
-                            # uring re-write this variable is global allow
-                            # direct access to INI structure
+                            # during re-write this variable is global
+                            # to allow direct access to INI structure
 
 database = {}               # Holds the option dictionary
 
 modified = False            # Signals a change in option dictionary
                             # Should be reset after saving to settings file
 
+paramfinder = re.compile(r'''(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)''')
+
 
 class Option(object):
     """ Basic option class, basic fields """
-    def __init__(self, section, keyword, default_val=None, add=True):
+
+    def __init__(self, section, keyword, default_val=None, add=True, protect=False):
         """ Basic option
-            section     : single section or comma-separated list of sections
-                          a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
-            keyword     : keyword in the (last) section
-            default_val : value returned when no value has been set
-            callback    : procedure to call when value is succesfully changed
+            `section`     : single section or comma-separated list of sections
+                            a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
+            `keyword`     : keyword in the (last) section
+            `default_val` : value returned when no value has been set
+            `callback`    : procedure to call when value is successfully changed
+            `protect`     : Do not allow setting via the API (specifically set_dict)
         """
         self.__sections = section.split(',')
         self.__keyword = keyword
         self.__default_val = default_val
         self.__value = None
         self.__callback = None
+        self.__protect = protect
 
         # Add myself to the config dictionary
         if add:
@@ -75,7 +84,7 @@ class Option(object):
 
     def get(self):
         """ Retrieve value field """
-        if self.__value != None:
+        if self.__value is not None:
             return self.__value
         else:
             return self.__default_val
@@ -85,10 +94,12 @@ class Option(object):
 
     def get_dict(self, safe=False):
         """ Return value a dictionary """
-        return { self.__keyword : self.get() }
+        return {self.__keyword: self.get()}
 
     def set_dict(self, dict):
         """ Set value based on dictionary """
+        if self.__protect:
+            return False
         try:
             return self.set(dict['value'])
         except KeyError:
@@ -97,8 +108,8 @@ class Option(object):
     def __set(self, value):
         """ Set new value, no validation """
         global modified
-        if (value != None):
-            if type(value) == type([]) or type(value) == type({}) or value != self.__value:
+        if value is not None:
+            if isinstance(value, list) or isinstance(value, dict) or value != self.__value:
                 self.__value = value
                 modified = True
                 if self.__callback:
@@ -120,19 +131,19 @@ class Option(object):
         return self.__sections, self.__keyword
 
 
-
 class OptionNumber(Option):
     """ Numeric option class, int/float is determined from default value """
-    def __init__(self, section, keyword, default_val=0, minval=None, maxval=None, validation=None, add=True):
-        Option.__init__(self, section, keyword, default_val, add=add)
+
+    def __init__(self, section, keyword, default_val=0, minval=None, maxval=None, validation=None, add=True, protect=False):
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
         self.__minval = minval
         self.__maxval = maxval
         self.__validation = validation
-        self.__int = type(default_val) == type(0)
+        self.__int = isinstance(default_val, int)
 
     def set(self, value):
         """ set new value, limited by range """
-        if value != None:
+        if value is not None:
             try:
                 if self.__int:
                     value = int(value)
@@ -144,9 +155,9 @@ class OptionNumber(Option):
                 error, val = self.__validation(value)
                 self._Option__set(val)
             else:
-                if (self.__maxval != None) and value > self.__maxval:
+                if self.__maxval is not None and value > self.__maxval:
                     value = self.__maxval
-                elif (self.__minval != None) and value < self.__minval:
+                elif self.__minval is not None and value < self.__minval:
                     value = self.__minval
                 self._Option__set(value)
         return None
@@ -154,8 +165,9 @@ class OptionNumber(Option):
 
 class OptionBool(Option):
     """ Boolean option class """
-    def __init__(self, section, keyword, default_val=False, add=True):
-        Option.__init__(self, section, keyword, int(default_val), add=add)
+
+    def __init__(self, section, keyword, default_val=False, add=True, protect=False):
+        Option.__init__(self, section, keyword, int(default_val), add=add, protect=protect)
 
     def set(self, value):
         if value is None:
@@ -169,13 +181,26 @@ class OptionBool(Option):
 
 class OptionDir(Option):
     """ Directory option class """
-    def __init__(self, section, keyword, default_val='', apply_umask=False, create=True, validation=None, add=True):
+
+    def __init__(self, section, keyword, default_val='', apply_umask=False, create=True, validation=None, writable=True, add=True):
         self.__validation = validation
         self.__root = ''   # Base directory for relative paths
         self.__apply_umask = apply_umask
         self.__create = create
+        self.__writable = writable
         Option.__init__(self, section, keyword, default_val, add=add)
 
+    def get(self):
+        """ Return value, corrected for platform """
+        if self._Option__value is not None:
+            p = self._Option__value
+        else:
+            p = self._Option__default_val
+        if sabnzbd.WIN32:
+            return p.replace('/', '\\') if '/' in p else p
+        else:
+            return p.replace('\\', '/') if '\\' in p else p
+
     def get_path(self):
         """ Return full absolute path """
         value = self.get()
@@ -183,7 +208,7 @@ class OptionDir(Option):
         if value:
             path = sabnzbd.misc.real_path(self.__root, value)
             if self.__create and not os.path.exists(path):
-                res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
+                res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
         return path
 
     def test_path(self):
@@ -205,15 +230,15 @@ class OptionDir(Option):
             'create' means try to create (but don't set permanent create flag)
         """
         error = None
-        if value != None and (create or value != self.get()):
+        if value is not None and (create or value != self.get()):
             value = value.strip()
             if self.__validation:
                 error, value = self.__validation(self.__root, value, self._Option__default_val)
             if not error:
                 if value and (self.__create or create):
-                    res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
+                    res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
                     if not res:
-                        error = Ta("Cannot create %s folder %s") % (self.ident()[1], path)
+                        error = T('Cannot create %s folder %s') % (self.ident()[1], path)
             if not error:
                 self._Option__set(value)
         return error
@@ -225,21 +250,22 @@ class OptionDir(Option):
 
 class OptionList(Option):
     """ List option class """
-    def __init__(self, section, keyword, default_val=None, validation=None, add=True):
+
+    def __init__(self, section, keyword, default_val=None, validation=None, add=True, protect=False):
         self.__validation = validation
         if default_val is None:
             default_val = []
-        Option.__init__(self, section, keyword, default_val, add=add)
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
 
     def set(self, value):
-        """ Set the list given a comma-separated string or a list"""
+        """ Set the list given a comma-separated string or a list """
         error = None
         if value is not None:
             if not isinstance(value, list):
                 if '"' not in value and ',' not in value:
                     value = value.split()
                 else:
-                    value = listquote.simplelist(value)
+                    value = paramfinder.findall(value)
             if self.__validation:
                 error, value = self.__validation(value)
             if not error:
@@ -249,20 +275,25 @@ class OptionList(Option):
     def get_string(self):
         """ Return the list as a comma-separated string """
         lst = self.get()
-        if type(lst) == type(""):
+        if isinstance(lst, basestring):
             return lst
-        txt = ''
-        r = len(lst)
-        for n in xrange(r):
-            txt += lst[n]
-            if n < r-1: txt += ', '
-        return txt
+        else:
+            return ', '.join(lst)
+
+    def default_string(self):
+        """ Return the default list as a comma-separated string """
+        lst = self.default()
+        if isinstance(lst, basestring):
+            return lst
+        else:
+            return ', '.join(lst)
 
 
 class OptionStr(Option):
     """ String class """
-    def __init__(self, section, keyword, default_val='', validation=None, add=True, strip=True):
-        Option.__init__(self, section, keyword, default_val, add=add)
+
+    def __init__(self, section, keyword, default_val='', validation=None, add=True, strip=True, protect=False):
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
         self.__validation = validation
         self.__strip = strip
 
@@ -277,7 +308,7 @@ class OptionStr(Option):
     def set(self, value):
         """ Set stripped value """
         error = None
-        if type(value) == type('') and self.__strip:
+        if isinstance(value, basestring) and self.__strip:
             value = value.strip()
         if self.__validation:
             error, val = self.__validation(value)
@@ -289,6 +320,7 @@ class OptionStr(Option):
 
 class OptionPassword(Option):
     """ Password class """
+
     def __init__(self, section, keyword, default_val='', add=True):
         Option.__init__(self, section, keyword, default_val, add=add)
         self.get_string = self.get_stars
@@ -308,20 +340,20 @@ class OptionPassword(Option):
     def get_dict(self, safe=False):
         """ Return value a dictionary """
         if safe:
-            return { self._Option__keyword : self.get_stars() }
+            return {self._Option__keyword: self.get_stars()}
         else:
-            return { self._Option__keyword : self.get() }
+            return {self._Option__keyword: self.get()}
 
     def set(self, pw):
         """ Set password, encode it """
-        if (pw != None and pw == '') or (pw and pw.strip('*')):
+        if (pw is not None and pw == '') or (pw and pw.strip('*')):
             self._Option__set(encode_password(pw))
         return None
 
 
 @synchronized(CONFIG_LOCK)
 def add_to_database(section, keyword, obj):
-    """ add object as secion/keyword to INI database """
+    """ add object as section/keyword to INI database """
     global database
     if section not in database:
         database[section] = {}
@@ -344,41 +376,50 @@ def delete_from_database(section, keyword):
 
 class ConfigServer(object):
     """ Class defining a single server """
+
     def __init__(self, name, values):
 
         self.__name = name
         name = 'servers,' + self.__name
 
+        self.displayname = OptionStr(name, 'displayname', '', add=False)
         self.host = OptionStr(name, 'host', '', add=False)
-        self.port = OptionNumber(name, 'port', 119, 0, 2**16-1, add=False)
-        self.timeout = OptionNumber(name, 'timeout', 120, 30, 240, add=False)
+        self.port = OptionNumber(name, 'port', 119, 0, 2 ** 16 - 1, add=False)
+        self.timeout = OptionNumber(name, 'timeout', 60, 20, 240, add=False)
         self.username = OptionStr(name, 'username', '', add=False)
         self.password = OptionPassword(name, 'password', '', add=False)
         self.connections = OptionNumber(name, 'connections', 1, 0, 100, add=False)
-        self.fillserver = OptionBool(name, 'fillserver', False, add=False)
         self.ssl = OptionBool(name, 'ssl', False, add=False)
+        self.ssl_verify = OptionNumber(name, 'ssl_verify', 2, add=False)  # 0=No, 1=Normal, 2=Strict (hostname verification)
+        self.ssl_ciphers = OptionStr(name, 'ssl_ciphers', '', add=False)
         self.enable = OptionBool(name, 'enable', True, add=False)
         self.optional = OptionBool(name, 'optional', False, add=False)
         self.retention = OptionNumber(name, 'retention', add=False)
+        self.send_group = OptionBool(name, 'send_group', False, add=False)
+        self.priority = OptionNumber(name, 'priority', 0, 0, 99, add=False)
+        self.notes = OptionStr(name, 'notes', '', add=False)
 
         self.set_dict(values)
         add_to_database('servers', self.__name, self)
 
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
-        for kw in ('host', 'port', 'timeout', 'username', 'password', 'connections',
-                   'fillserver', 'ssl', 'enable', 'optional', 'retention'):
+        for kw in ('displayname', 'host', 'port', 'timeout', 'username', 'password', 'connections', 'ssl',
+                   'ssl_verify', 'ssl_ciphers', 'send_group', 'enable', 'optional', 'retention', 'priority', 'notes'):
             try:
                 value = values[kw]
             except KeyError:
                 continue
             exec 'self.%s.set(value)' % kw
+            if not self.displayname():
+                self.displayname.set(self.__name)
         return True
 
     def get_dict(self, safe=False):
         """ Return a dictionary with all attributes """
         dict = {}
         dict['name'] = self.__name
+        dict['displayname'] = self.displayname()
         dict['host'] = self.host()
         dict['port'] = self.port()
         dict['timeout'] = self.timeout()
@@ -388,11 +429,15 @@ class ConfigServer(object):
         else:
             dict['password'] = self.password()
         dict['connections'] = self.connections()
-        dict['fillserver'] = self.fillserver()
         dict['ssl'] = self.ssl()
+        dict['ssl_verify'] = self.ssl_verify()
+        dict['ssl_ciphers'] = self.ssl_ciphers()
         dict['enable'] = self.enable()
         dict['optional'] = self.optional()
         dict['retention'] = self.retention()
+        dict['send_group'] = self.send_group()
+        dict['priority'] = self.priority()
+        dict['notes'] = self.notes()
         return dict
 
     def delete(self):
@@ -400,10 +445,8 @@ class ConfigServer(object):
         delete_from_database('servers', self.__name)
 
     def rename(self, name):
-        """ Give server new identity """
-        delete_from_database('servers', self.__name)
-        self.__name = name
-        add_to_database('servers', self.__name, self)
+        """ Give server new display name """
+        self.displayname.set(name)
 
     def ident(self):
         return 'servers', self.__name
@@ -411,10 +454,12 @@ class ConfigServer(object):
 
 class ConfigCat(object):
     """ Class defining a single category """
+
     def __init__(self, name, values):
         self.__name = name
         name = 'categories,' + name
 
+        self.order = OptionNumber(name, 'order', 0, 0, 100, add=False)
         self.pp = OptionStr(name, 'pp', '', add=False)
         self.script = OptionStr(name, 'script', 'Default', add=False)
         self.dir = OptionDir(name, 'dir', add=False, create=False)
@@ -426,7 +471,7 @@ class ConfigCat(object):
 
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
-        for kw in ('pp', 'script', 'dir', 'newzbin', 'priority'):
+        for kw in ('order', 'pp', 'script', 'dir', 'newzbin', 'priority'):
             try:
                 value = values[kw]
             except KeyError:
@@ -438,6 +483,7 @@ class ConfigCat(object):
         """ Return a dictionary with all attributes """
         dict = {}
         dict['name'] = self.__name
+        dict['order'] = self.order()
         dict['pp'] = self.pp()
         dict['script'] = self.script()
         dict['dir'] = self.dir()
@@ -452,6 +498,7 @@ class ConfigCat(object):
 
 class OptionFilters(Option):
     """ Filter list class """
+
     def __init__(self, section, keyword, add=True):
         Option.__init__(self, section, keyword, add=add)
         self.set([])
@@ -491,7 +538,7 @@ class OptionFilters(Option):
         dict = {}
         n = 0
         for filter in self.get():
-            dict['filter'+str(n)] = filter
+            dict['filter' + str(n)] = filter
             n = n + 1
         return dict
 
@@ -503,10 +550,10 @@ class OptionFilters(Option):
             val = values.get(kw)
             if val is not None:
                 val = values[kw]
-                if type(val) == type([]):
+                if isinstance(val, list):
                     filters.append(val)
                 else:
-                    filters.append(listquote.simplelist(val))
+                    filters.append(paramfinder.findall(val))
                 while len(filters[-1]) < 7:
                     filters[-1].append('1')
                 if not filters[-1][6]:
@@ -515,13 +562,15 @@ class OptionFilters(Option):
             self.set(filters)
         return True
 
+
 class ConfigRSS(object):
     """ Class defining a single Feed definition """
+
     def __init__(self, name, values):
         self.__name = name
         name = 'rss,' + name
 
-        self.uri = OptionStr(name, 'uri', add=False)
+        self.uri = OptionList(name, 'uri', add=False)
         self.cat = OptionStr(name, 'cat', add=False)
         self.pp = OptionStr(name, 'pp', '', add=False)
         self.script = OptionStr(name, 'script', add=False)
@@ -568,9 +617,8 @@ class ConfigRSS(object):
         return 'rss', self.__name
 
 
-
 def get_dconfig(section, keyword, nested=False):
-    """ Return a config values dictonary,
+    """ Return a config values dictionary,
         Single item or slices based on 'section', 'keyword'
     """
     data = {}
@@ -603,16 +651,15 @@ def get_dconfig(section, keyword, nested=False):
         data = item.get_dict(safe=True)
         if not nested:
             if section in ('servers', 'categories', 'rss'):
-                data = {section : [ data ]}
+                data = {section: [data]}
             else:
-                data = {section : data}
+                data = {section: data}
 
     return True, data
 
 
 def get_config(section, keyword):
-    """ Return a config object, based on 'section', 'keyword'
-    """
+    """ Return a config object, based on 'section', 'keyword' """
     try:
         return database[section][keyword]
     except KeyError:
@@ -621,8 +668,7 @@ def get_config(section, keyword):
 
 
 def set_config(kwargs):
-    """ Set a config item, using values in dictionary
-    """
+    """ Set a config item, using values in dictionary """
     try:
         item = database[kwargs.get('section')][kwargs.get('keyword')]
     except KeyError:
@@ -632,21 +678,19 @@ def set_config(kwargs):
 
 
 def delete(section, keyword):
-    """ Delete specific config item
-    """
+    """ Delete specific config item """
     try:
         database[section][keyword].delete()
     except KeyError:
         return
 
 
-################################################################################
-#
+##############################################################################
 # INI file support
 #
 # This does input and output of configuration to an INI file.
 # It translates this data structure to the config database.
-
+##############################################################################
 @synchronized(SAVE_CONFIG_LOCK)
 def read_config(path):
     """ Read the complete INI file and check its version number
@@ -683,25 +727,45 @@ def _read_config(path, try_backup=False):
             return False, 'Cannot create INI file %s' % path
 
     try:
-        CFG = configobj.ConfigObj(path)
+        fp = open(path, 'rb')
+        lines = fp.read().split('\n')
+        if len(lines) == 1:
+            fp.seek(0)
+            lines = fp.read().split('\r')
+        lines = [line.rstrip('\r\n') for line in lines]
+        fp.close()
+
         try:
-            if int(CFG['__version__']) > int(CONFIG_VERSION):
-                return False, "Incorrect version number %s in %s" % (CFG['__version__'], path)
-        except (KeyError, ValueError):
-            CFG['__version__'] = CONFIG_VERSION
-    except configobj.ConfigObjError, strerror:
+            # First try UTF-8 encoding
+            CFG = configobj.ConfigObj(lines, default_encoding='utf-8', encoding='utf-8')
+        except UnicodeDecodeError:
+            # Failed, enable retry
+            CFG = {}
+
+        if not re.search(r'utf[ -]*8', CFG.get('__encoding__', ''), re.I):
+            # INI file is still in 8bit ASCII encoding, so try Latin-1 instead
+            CFG = configobj.ConfigObj(lines, default_encoding='cp1252', encoding='cp1252')
+
+    except (IOError, configobj.ConfigObjError, UnicodeEncodeError), strerror:
         if try_backup:
+            if isinstance(strerror, UnicodeEncodeError):
+                strerror = 'Character encoding of the file is inconsistent'
             return False, '"%s" is not a valid configuration file<br>Error message: %s' % (path, strerror)
         else:
+            # Try backup file
             return _read_config(path, True)
 
-    CFG['__version__'] = CONFIG_VERSION
+    try:
+        version = sabnzbd.misc.int_conv(CFG['__version__'])
+        if version > int(CONFIG_VERSION):
+            return False, "Incorrect version number %s in %s" % (version, path)
+    except (KeyError, ValueError):
+        pass
 
-    if 'misc' in CFG:
-        compatibility_fix(CFG['misc'])
-
-    if 'rss' in CFG:
-        newzbin_fix(CFG['rss'])
+    CFG.filename = path
+    CFG.encoding = 'utf-8'
+    CFG['__encoding__'] = u'utf-8'
+    CFG['__version__'] = unicode(CONFIG_VERSION)
 
     # Use CFG data to set values for all static options
     for section in database:
@@ -722,12 +786,10 @@ def _read_config(path, try_backup=False):
     return True, ""
 
 
-
 @synchronized(SAVE_CONFIG_LOCK)
 def save_config(force=False):
     """ Update Setup file with current option values """
     global CFG, database, modified
-    assert isinstance(CFG, configobj.ConfigObj)
 
     if not (modified or force):
         return True
@@ -740,7 +802,7 @@ def save_config(force=False):
                 CFG[section] = {}
             for subsec in database[section]:
                 if section == 'servers':
-                    subsec_mod = subsec.replace('[', '{').replace(']','}')
+                    subsec_mod = subsec.replace('[', '{').replace(']', '}')
                 else:
                     subsec_mod = subsec
                 try:
@@ -758,9 +820,11 @@ def save_config(force=False):
                 except KeyError:
                     CFG[sec] = {}
                 value = database[section][option]()
-                if type(value) == type(True):
+                # bool is a subclass of int, check first
+                if isinstance(value, bool):
+                    # convert bool to int when saving so we store 0 or 1
                     CFG[sec][kw] = str(int(value))
-                elif type(value) == type(0):
+                elif isinstance(value, int):
                     CFG[sec][kw] = str(value)
                 else:
                     CFG[sec][kw] = value
@@ -771,30 +835,39 @@ def save_config(force=False):
 
     # Check if file is writable
     if not sabnzbd.misc.is_writable(filename):
-        logging.error(Ta('Cannot write to INI file %s'), filename)
+        logging.error(T('Cannot write to INI file %s'), filename)
         return res
 
     # copy current file to backup
     try:
         shutil.copyfile(filename, bakname)
+        shutil.copymode(filename, bakname)
     except:
         # Something wrong with the backup,
-        logging.warning(Ta('Cannot create backup file for %s'), bakname)
-        logging.info("Traceback: ", exc_info = True)
+        logging.error(T('Cannot create backup file for %s'), bakname)
+        logging.info("Traceback: ", exc_info=True)
+        return res
 
     # Write new config file
     try:
+        logging.info('Writing settings to INI file %s', filename)
         CFG.write()
+        shutil.copymode(bakname, filename)
         modified = False
         res = True
     except:
-        logging.error(Ta('Cannot write to INI file %s'), filename)
-        logging.info("Traceback: ", exc_info = True)
+        logging.error(T('Cannot write to INI file %s'), filename)
+        logging.info("Traceback: ", exc_info=True)
+        try:
+            sabnzbd.misc.remove_file(filename)
+        except:
+            pass
+        # Restore INI file from backup
+        sabnzbd.misc.renamer(bakname, filename)
 
     return res
 
 
-
 def define_servers():
     """ Define servers listed in the Setup file
         return a list of ConfigServer instances
@@ -803,10 +876,18 @@ def define_servers():
     try:
         for server in CFG['servers']:
             svr = CFG['servers'][server]
-            ConfigServer(server.replace('{', '[').replace('}', ']'), svr)
+            s = ConfigServer(server.replace('{', '[').replace('}', ']'), svr)
+
+            # Conversion of global SSL-Ciphers to server ones
+            if sabnzbd.cfg.ssl_ciphers():
+                s.ssl_ciphers.set(sabnzbd.cfg.ssl_ciphers())
     except KeyError:
         pass
 
+    # No longer needed
+    sabnzbd.cfg.ssl_ciphers.set('')
+
+
 def get_servers():
     global database
     try:
@@ -844,9 +925,18 @@ def get_categories(cat=0):
     if 'categories' not in database:
         database['categories'] = {}
     cats = database['categories']
+
+    # Add Default categories
     if '*' not in cats:
-        ConfigCat('*', {'pp' : old_def('dirscan_opts', '3'), 'script' : old_def('dirscan_script', 'None'), \
-                        'priority' : old_def('dirscan_priority', NORMAL_PRIORITY)})
+        ConfigCat('*', {'pp': old_def('dirscan_opts', '3'), 'script': old_def('dirscan_script', 'None'),
+                        'priority': old_def('dirscan_priority', NORMAL_PRIORITY)})
+        # Add some category suggestions
+        ConfigCat('movies', {})
+        ConfigCat('tv', {})
+        ConfigCat('audio', {})
+        ConfigCat('software', {})
+
+        # Save config for future use
         save_config(True)
     if not isinstance(cat, int):
         try:
@@ -856,6 +946,25 @@ def get_categories(cat=0):
     return cats
 
 
+def get_ordered_categories():
+    """ Return list-copy of categories section that's ordered
+        by user's ordering including Default-category
+    """
+    database_cats = get_categories()
+
+    # Transform to list and sort
+    categories = []
+    for cat in database_cats.keys():
+        if cat != '*':
+            categories.append(database_cats[cat].get_dict())
+
+    # Sort and add default * category
+    categories.sort(key=lambda cat: cat['order'])
+    categories.insert(0, database_cats['*'].get_dict())
+
+    return categories
+
+
 def define_rss():
     """ Define rss-feeds listed in the Setup file
         return a list of ConfigRSS instances
@@ -867,25 +976,43 @@ def define_rss():
     except KeyError:
         pass
 
+
 def get_rss():
     global database
     try:
+        # We have to remove non-seperator commas by detecting if they are valid URL's
+        for feed_key in database['rss']:
+            feed = database['rss'][feed_key]
+            # Only modify if we have to, to prevent repeated config-saving
+            have_new_uri = False
+            # Create a new corrected list
+            new_feed_uris = []
+            for feed_uri in feed.uri():
+                if new_feed_uris and not urlparse(feed_uri).scheme and urlparse(new_feed_uris[-1]).scheme:
+                    # Current one has no scheme but previous one does, append to previous
+                    new_feed_uris[-1] += '%2C' + feed_uri
+                    have_new_uri = True
+                    continue
+                # Add full working URL
+                new_feed_uris.append(feed_uri)
+            # Set new list
+            if have_new_uri:
+                feed.uri.set(new_feed_uris)
+
         return database['rss']
     except KeyError:
         return {}
 
+
 def get_filename():
     global CFG
     return CFG.filename
 
 
-################################################################################
-#
+##############################################################################
 # Default Validation handlers
-#
+##############################################################################
 __PW_PREFIX = '!!!encoded!!!'
-
-#------------------------------------------------------------------------------
 def encode_password(pw):
     """ Encode password in hexadecimal if needed """
     enc = False
@@ -909,9 +1036,9 @@ def decode_password(pw, name):
     if pw and pw.startswith(__PW_PREFIX):
         for n in range(len(__PW_PREFIX), len(pw), 2):
             try:
-                ch = chr( int(pw[n] + pw[n+1], 16) )
+                ch = chr(int(pw[n] + pw[n + 1], 16))
             except ValueError:
-                logging.error(Ta('Incorrectly encoded password %s'), name)
+                logging.error(T('Incorrectly encoded password %s'), name)
                 return ''
             decPW += ch
         return decPW
@@ -927,6 +1054,14 @@ def no_nonsense(value):
     return None, value
 
 
+def all_lowercase(value):
+    """ Lowercase everything! """
+    if isinstance(value, list):
+        # If list, for each item
+        return None, [item.lower() for item in value]
+    return None, value.lower()
+
+
 def validate_octal(value):
     """ Check if string is valid octal number """
     if not value:
@@ -935,7 +1070,7 @@ def validate_octal(value):
         int(value, 8)
         return None, value
     except:
-        return Ta('%s is not a correct octal value') % value, None
+        return T('%s is not a correct octal value') % value, None
 
 
 def validate_no_unc(root, value, default):
@@ -944,24 +1079,19 @@ def validate_no_unc(root, value, default):
     if value and not value.startswith(r'\\'):
         return validate_notempty(root, value, default)
     else:
-        return Ta('UNC path "%s" not allowed here') % value, None
+        return T('UNC path "%s" not allowed here') % value, None
 
 
 def validate_safedir(root, value, default):
-    """ Allow only when queues are empty and no UNC """
+    """ Allow only when queues are empty and no UNC
+        On Windows path should be small
+    """
+    if sabnzbd.WIN32 and value and len(sabnzbd.misc.real_path(root, value)) >= MAX_WIN_DFOLDER:
+        return T('Error: Path length should be below %s.') % MAX_WIN_DFOLDER, None
     if sabnzbd.empty_queues():
         return validate_no_unc(root, value, default)
     else:
-        return Ta('Error: Queue not empty, cannot change folder.'), None
-
-
-def validate_dir_exists(root, value, default):
-    """ Check if directory exists """
-    p = sabnzbd.misc.real_path(root, value)
-    if os.path.exists(p):
-        return None, value
-    else:
-        return Ta('Folder "%s" does not exist') % p, None
+        return T('Error: Queue not empty, cannot change folder.'), None
 
 
 def validate_notempty(root, value, default):
@@ -973,14 +1103,7 @@ def validate_notempty(root, value, default):
 
 
 def create_api_key():
-    """ Return a new randomized API_KEY
-    """
-    import time
-    try:
-        from hashlib import md5
-    except ImportError:
-        from md5 import md5
-    import random
+    """ Return a new randomized API_KEY """
     # Create some values to seed md5
     t = str(time.time())
     r = str(random.random())
@@ -991,35 +1114,3 @@ def create_api_key():
 
     # Return a hex digest of the md5, eg 49f68a5c8493ec2c0bf489821c21fc3b
     return m.hexdigest()
-
-
-#------------------------------------------------------------------------------
-_FIXES = \
-(
-    ('bandwith_limit', 'bandwidth_limit'),
-    ('enable_par_multicore', 'par2_multicore')
-)
-
-def compatibility_fix(cf):
-    """ Convert obsolete INI entries """
-    for item in _FIXES:
-        old, new = item
-        try:
-            cf[new]
-        except KeyError:
-            try:
-                cf[new] = cf[old]
-                del cf[old]
-            except KeyError:
-                pass
-    try:
-        cf['rating_host'] = cf['rating_host'].replace('www.oznzb.com', 'api.oznzb.com')
-    except KeyError:
-        pass
-
-def newzbin_fix(cf):
-    """ Replace old newzbin links """
-    for feed in cf:
-        item = cf[feed].get('uri')
-        if item and 'newzbin.com' in item:
-            cf[feed]['uri'] = item.replace('newzbin.com', 'newzbin2.es')
