@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -33,7 +33,7 @@ from sabnzbd.encoding import TRANS, unicoder, platform_encode, deunicode
 import sabnzbd.utils.rarfile as rarfile
 from sabnzbd.misc import format_time_string, find_on_path, make_script_path, int_conv, \
     real_path, globber, globber_full, get_all_passwords, renamer, clip_path, \
-    has_win_device, calc_age, long_path, remove_file
+    has_win_device, calc_age, long_path, remove_file, recursive_listdir
 from sabnzbd.sorting import SeriesSorter
 import sabnzbd.cfg as cfg
 from sabnzbd.constants import Status
@@ -41,8 +41,12 @@ from sabnzbd.constants import Status
 if sabnzbd.WIN32:
     try:
         import win32api
-        from win32con import SW_HIDE
-        from win32process import STARTF_USESHOWWINDOW, IDLE_PRIORITY_CLASS
+        import win32con
+        import win32process
+
+        # Define scheduling priorities
+        WIN_SCHED_PRIOS = {1: win32process.IDLE_PRIORITY_CLASS, 2: win32process.BELOW_NORMAL_PRIORITY_CLASS,
+                           3: win32process.NORMAL_PRIORITY_CLASS, 4: win32process.ABOVE_NORMAL_PRIORITY_CLASS,}
 
         # Use patched version of subprocess module for Unicode on Windows
         import subprocessww
@@ -61,14 +65,14 @@ else:
 from subprocess import Popen
 
 # Regex globals
-RAR_RE = re.compile(r'\.(?P<ext>part\d*\.rar|rar|r\d\d|s\d\d|t\d\d|u\d\d|v\d\d|\d\d\d)$', re.I)
+RAR_RE = re.compile(r'\.(?P<ext>part\d*\.rar|rar|r\d\d|s\d\d|t\d\d|u\d\d|v\d\d|\d\d\d?\d)$', re.I)
 RAR_RE_V3 = re.compile(r'\.(?P<ext>part\d*)$', re.I)
 
 LOADING_RE = re.compile(r'^Loading "(.+)"')
 TARGET_RE = re.compile(r'^(?:File|Target): "(.+)" -')
 EXTRACTFROM_RE = re.compile(r'^Extracting\sfrom\s(.+)')
 EXTRACTED_RE = re.compile(r'^(Extracting|Creating|...)\s+(.*?)\s+OK\s*$')
-SPLITFILE_RE = re.compile(r'\.(\d\d\d$)', re.I)
+SPLITFILE_RE = re.compile(r'\.(\d\d\d?\d$)', re.I)
 ZIP_RE = re.compile(r'\.(zip$)', re.I)
 SEVENZIP_RE = re.compile(r'\.7z$', re.I)
 SEVENMULTI_RE = re.compile(r'\.7z\.\d+$', re.I)
@@ -242,7 +246,7 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zi
     rerun = False
     force_rerun = False
     newfiles = []
-    error = 0
+    error = None
     new_joins = new_rars = new_zips = new_ts = None
 
     if cfg.enable_filejoin():
@@ -253,8 +257,6 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zi
             if newf:
                 newfiles.extend(newf)
             logging.info('Filejoin finished on %s', workdir)
-            nzo.set_action_line()
-            rerun = not error
 
     if cfg.enable_unrar():
         new_rars = [rar for rar in xrars if rar not in rars]
@@ -264,32 +266,27 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zi
             if newf:
                 newfiles.extend(newf)
             logging.info('Unrar finished on %s', workdir)
-            nzo.set_action_line()
-            rerun = not error
 
     if cfg.enable_7zip():
         new_sevens = [seven for seven in xsevens if seven not in sevens]
         if new_sevens:
             logging.info('7za starting on %s', workdir)
-            if unseven(nzo, workdir, workdir_complete, dele, one_folder, new_sevens):
-                error = True
+            error, newf = unseven(nzo, workdir, workdir_complete, dele, one_folder, new_sevens)
+            if newf:
+                newfiles.extend(newf)
             logging.info('7za finished on %s', workdir)
-            nzo.set_action_line()
-            rerun = not error
 
     if cfg.enable_unzip():
         new_zips = [zip for zip in xzips if zip not in zips]
         if new_zips:
             logging.info('Unzip starting on %s', workdir)
             if SEVEN_COMMAND:
-                if unseven(nzo, workdir, workdir_complete, dele, one_folder, new_zips):
-                    error = True
+                error, newf = unseven(nzo, workdir, workdir_complete, dele, one_folder, new_zips)
             else:
-                if unzip(nzo, workdir, workdir_complete, dele, one_folder, new_zips):
-                    error = True
+                error, newf = unzip(nzo, workdir, workdir_complete, dele, one_folder, new_zips)
+            if newf:
+                newfiles.extend(newf)
             logging.info('Unzip finished on %s', workdir)
-            nzo.set_action_line()
-            rerun = not error
 
     if cfg.enable_tsjoin():
         new_ts = [_ts for _ts in xts if _ts not in ts]
@@ -299,8 +296,12 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zi
             if newf:
                 newfiles.extend(newf)
             logging.info('TS Joining finished on %s', workdir)
-            nzo.set_action_line()
-            rerun = not error
+
+    # Refresh history and set output
+    nzo.set_action_line()
+
+    # Only re-run if something was unpacked and it was success
+    rerun = error in (False, 0)
 
     # During a Retry we might miss files that failed during recursive unpack
     if nzo.reuse and depth == 1 and any(build_filelists(workdir, workdir_complete)):
@@ -849,6 +850,9 @@ def unzip(nzo, workdir, workdir_complete, delete, one_folder, zips):
         unzip_failed = False
         tms = time.time()
 
+        # For file-bookkeeping
+        orig_dir_content = recursive_listdir(workdir_complete)
+
         for _zip in zips:
             logging.info("Starting extract on zipfile: %s ", _zip)
             nzo.set_action_line(T('Unpacking'), '%s' % unicoder(os.path.basename(_zip)))
@@ -866,6 +870,9 @@ def unzip(nzo, workdir, workdir_complete, delete, one_folder, zips):
         msg = T('%s files in %s') % (str(i), format_time_string(time.time() - tms))
         nzo.set_unpack_info('Unpack', msg)
 
+        # What's new?
+        new_files = list(set(orig_dir_content + recursive_listdir(workdir_complete)))
+
         # Delete the old files if we have to
         if delete and not unzip_failed:
             i = 0
@@ -886,12 +893,12 @@ def unzip(nzo, workdir, workdir_complete, delete, one_folder, zips):
                     except OSError:
                         logging.warning(T('Deleting %s failed!'), brokenzip)
 
-        return unzip_failed
+        return unzip_failed, new_files
     except:
         msg = sys.exc_info()[1]
         nzo.fail_msg = T('Unpacking failed, %s') % msg
         logging.error(T('Error "%s" while running unzip() on %s'), msg, nzo.final_name)
-        return True
+        return True, []
 
 
 def ZIP_Extract(zipfile, extraction_path, one_folder):
@@ -925,6 +932,7 @@ def unseven(nzo, workdir, workdir_complete, delete, one_folder, sevens):
     """
     i = 0
     unseven_failed = False
+    new_files = []
     tms = time.time()
 
     # Find multi-volume sets, because 7zip will not provide actual set members
@@ -951,18 +959,19 @@ def unseven(nzo, workdir, workdir_complete, delete, one_folder, sevens):
         else:
             extraction_path = os.path.split(seven)[0]
 
-        res, msg = seven_extract(nzo, seven, extensions, extraction_path, one_folder, delete)
+        res, new_files_set, msg = seven_extract(nzo, seven, extensions, extraction_path, one_folder, delete)
         if res:
             unseven_failed = True
             nzo.set_unpack_info('Unpack', msg)
         else:
             i += 1
+        new_files.extend(new_files_set)
 
     if not unseven_failed:
         msg = T('%s files in %s') % (str(i), format_time_string(time.time() - tms))
         nzo.set_unpack_info('Unpack', msg)
 
-    return unseven_failed
+    return unseven_failed, new_files
 
 
 def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete):
@@ -978,7 +987,7 @@ def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete
             msg = T('Trying 7zip with password "%s"') % unicoder(password)
             nzo.fail_msg = msg
             nzo.set_unpack_info('Unpack', msg)
-        fail, msg = seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete, password)
+        fail, new_files, msg = seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete, password)
         if fail != 2:
             break
 
@@ -987,7 +996,7 @@ def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete
         msg = '%s (%s)' % (T('Unpacking failed, archive requires a password'), os.path.basename(sevenset))
         nzo.fail_msg = msg
         logging.error(msg)
-    return fail, msg
+    return fail, new_files, msg
 
 
 def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete, password):
@@ -1021,6 +1030,9 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
     if not os.path.exists(name):
         return 1, T('7ZIP set "%s" is incomplete, cannot unpack') % unicoder(sevenset)
 
+    # For file-bookkeeping
+    orig_dir_content = recursive_listdir(extraction_path)
+
     command = [SEVEN_COMMAND, method, '-y', overwrite, parm, case, password,
                '-o%s' % extraction_path, name]
 
@@ -1035,6 +1047,9 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
 
     ret = p.wait()
 
+    # What's new?
+    new_files = list(set(orig_dir_content + recursive_listdir(extraction_path)))
+
     if ret == 0 and delete:
         if extensions:
             for ext in extensions:
@@ -1050,7 +1065,7 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
                 logging.warning(T('Deleting %s failed!'), sevenset)
 
     # Always return an error message, even when return code is 0
-    return ret, T('Could not unpack %s') % unicoder(sevenset)
+    return ret, new_files, T('Could not unpack %s') % unicoder(sevenset)
 
 
 ##############################################################################
@@ -1842,10 +1857,23 @@ def MultiPar_Verify(parfile, parfile_nzf, nzo, setname, joinables, single=False)
             verifynum = 0
 
         elif in_repair:
-            # Line with percentage of repair (nothing else)
-            per = float(line[:-1])
-            nzo.set_action_line(T('Repairing'), '%2d%%' % per)
-            nzo.status = Status.REPAIRING
+            try:
+                # Line with percentage of repair (nothing else)
+                per = float(line[:-1])
+                nzo.set_action_line(T('Repairing'), '%2d%%' % per)
+                nzo.status = Status.REPAIRING
+            except:
+                # Checksum error
+                if 'checksum' in line:
+                    # Failed due to checksum error of multipar
+                    msg = T('Repairing failed, %s') % line
+                    nzo.fail_msg = msg
+                    msg = u'[%s] %s' % (unicoder(setname), msg)
+                    nzo.set_unpack_info('Repair', msg)
+                    nzo.status = Status.FAILED
+                else:
+                    # Not sure, log error
+                    logging.info("Traceback: ", exc_info=True)
 
         elif line.startswith('Repaired successfully'):
             msg = T('[%s] Repaired in %s') % (unicoder(setname), format_time_string(time.time() - start))
@@ -1994,9 +2022,9 @@ def build_command(command, flatten_command=False):
 
         need_shell = os.path.splitext(command[0])[1].lower() not in ('.exe', '.com')
         stup = subprocess.STARTUPINFO()
-        stup.dwFlags = STARTF_USESHOWWINDOW
-        stup.wShowWindow = SW_HIDE
-        creationflags = IDLE_PRIORITY_CLASS
+        stup.dwFlags = win32process.STARTF_USESHOWWINDOW
+        stup.wShowWindow = win32con.SW_HIDE
+        creationflags = WIN_SCHED_PRIOS[cfg.win_process_prio()]
 
         # Work-around for bug in Python's Popen function,
         # scripts with spaces in the path don't work.
@@ -2062,26 +2090,10 @@ def build_filelists(workdir, workdir_complete=None, check_both=False, check_rar=
     sevens, joinables, zips, rars, ts, filelist = ([], [], [], [], [], [])
 
     if workdir_complete:
-        for root, dirs, files in os.walk(workdir_complete):
-            for _file in files:
-                if '.AppleDouble' not in root and '.DS_Store' not in root:
-                    try:
-                        p = os.path.join(root, _file)
-                        filelist.append(p)
-                    except UnicodeDecodeError:
-                        # Just skip failing names
-                        pass
+        filelist.extend(recursive_listdir(workdir_complete))
 
     if workdir and (not filelist or check_both):
-        for root, dirs, files in os.walk(workdir):
-            for _file in files:
-                if '.AppleDouble' not in root and '.DS_Store' not in root:
-                    try:
-                        p = os.path.join(root, _file)
-                        filelist.append(p)
-                    except UnicodeDecodeError:
-                        # Just skip failing names
-                        pass
+        filelist.extend(recursive_listdir(workdir))
 
     for file in filelist:
         # Extra check for rar (takes CPU/disk)
