@@ -1,6 +1,6 @@
 #!/usr/bin/python -OO
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2012-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -270,6 +270,7 @@ SKIN_TEXT = {
 
 # Connections page
     'link-forceDisc' : TT('Force Disconnect'), #: Status page button
+    'explain-forceDisc' : TT('Disconnect all active connections to usenet servers. Connections will be reopened after a few seconds if there are items in the queue.'), #: Status page button text
     'askTestEmail' : TT('This will send a test email to your account.'),
     'link-showLog' : TT('Show Logging'), #: Status page button
     'link-testEmail' : TT('Test Email'), #: Status page button
@@ -476,6 +477,12 @@ SKIN_TEXT = {
     'explain-nice' : TT('Read the Wiki Help on this!'),
     'opt-ionice' : TT('IONice Parameters'),
     'explain-ionice' : TT('Read the Wiki Help on this!'),
+    'opt-win_process_prio' : TT('External process priority'),
+    'explain-win_process_prio' : TT('Read the Wiki Help on this!'),
+    'win_process_prio-high' : TT('High'),
+    'win_process_prio-normal' : TT('Normal'),
+    'win_process_prio-low' : TT('Low'),
+    'win_process_prio-idle' : TT('Idle'),
     'opt-auto_disconnect' : TT('Disconnect on Empty Queue'),
     'explain-auto_disconnect' : TT('Disconnect from Usenet server(s) when queue is empty or paused.'),
     'opt-auto_sort' : TT('Sort by Age'),
