@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -74,7 +74,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
         if not got_pass:
             return False, T('Password masked in ******, please re-enter')
     try:
-        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_verify, False, username, password)
+        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_verify, None, False, username, password)
     except:
         return False, T('Invalid server details')
 
@@ -119,8 +119,12 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
             nw.clear_data()
             nw.recv_chunk(block=True)
         except:
+            # Some internal error, not always safe to close connection
             return False, unicode(sys.exc_info()[1])
 
+    # Close the connection
+    nw.terminate(quit=True)
+
     if nw.status_code == '480':
         return False, T('Server requires username and password.')
 
@@ -136,5 +140,3 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
     else:
         return False, T('Could not determine connection result (%s)') % nntp_to_msg(nw.data)
 
-    # Close the connection
-    nw.terminate(quit=True)
