@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2009-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2009-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,9 +24,9 @@ import logging
 import os
 from sabnzbd.encoding import unicoder
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import get_ext, get_filename
+from sabnzbd.misc import get_ext, get_filename, get_from_url
 import sabnzbd.newsunpack
-from sabnzbd.constants import VALID_ARCHIVES
+from sabnzbd.constants import VALID_ARCHIVES, VALID_NZB_FILES
 
 from sabnzbd.dirscanner import ProcessArchiveFile, ProcessSingleFile
 
@@ -47,7 +47,7 @@ def upload_file(url, fp):
             password = cfg.password()
             if username and password:
                 url = '%s&ma_username=%s&ma_password=%s' % (url, username, password)
-        sabnzbd.newsunpack.get_from_url(url)
+        get_from_url(url)
     except:
         logging.error("Failed to upload file: %s", fp)
         logging.info("Traceback: ", exc_info=True)
@@ -60,7 +60,7 @@ def add_local(f):
         if fn:
             if get_ext(fn) in VALID_ARCHIVES:
                 ProcessArchiveFile(fn, f, keep=True)
-            elif get_ext(fn) in ('.nzb', '.gz', '.bz2'):
+            elif get_ext(fn) in VALID_NZB_FILES:
                 ProcessSingleFile(fn, f, keep=True)
         else:
             logging.error("Filename not found: %s", f)
